<?php
/**
 * Shortcode
 *
 * @package otfd
 * @subpackage _controllers
 * @since One Time File Download
 */

defined( 'ABSPATH' ) or die( 'No script kiddies please!' );

require_once OTFD_PLUGIN_PATH . '_controllers/class-main.php';
use Otfd\Controllers\Main as MainController;

require_once OTFD_PLUGIN_PATH . '_controllers/class-link.php';
use Otfd\Controllers\Link as LinkController;

require_once OTFD_PLUGIN_PATH . '_controllers/class-db.php';
use Otfd\Controllers\Db as DbController;

function otfd_curl_get_file_size( $url ) {
	if ( function_exists( 'curl_init' ) && function_exists( 'curl_exec' ) ) {
		// Assume failure.
		$result = 1;

		$curl = curl_init( $url );

		// Issue a HEAD request and follow any redirects.
		curl_setopt( $curl, CURLOPT_NOBODY, true );
		curl_setopt( $curl, CURLOPT_HEADER, true );
		curl_setopt( $curl, CURLOPT_RETURNTRANSFER, true );
		curl_setopt( $curl, CURLOPT_FOLLOWLOCATION, true );

		$data = curl_exec( $curl );
		curl_close( $curl );

		if( $data ) {
			$content_length = 1;
			$status = "unknown";

			if( preg_match( "/^HTTP\/1\.[01] (\d\d\d)/", $data, $matches ) ) {
				$status = (int)$matches[1];
			}

			if( preg_match( "/Content-Length: (\d+)/", $data, $matches ) ) {
				$content_length = (int)$matches[1];
			}

			// http://en.wikipedia.org/wiki/List_of_HTTP_status_codes
			if( $status == 200 || ($status > 300 && $status <= 308) ) {
				$result = $content_length;
			}
		}

		return $result;
	} elseif ( substr( $url, 0, 4 ) == 'http' ) {
		$headers = array_change_key_case( get_headers( $url, 1 ),CASE_LOWER );
		if ( strcasecmp( $headers[0], 'HTTP/1.1 200 OK' ) != 0 ) {
			$result = $headers['content-length'][1];
		} else {
			$result = $headers['content-length'];
		}
		return $result;
	} else {
		return -1;
	}
}

function check_if_wp_otfd_page() {
	if ( strpos( $_SERVER['REQUEST_URI'], OTFD_DOWNLOAD_PAGE ) !== false || isset( $_GET['wp_otfd_file'] ) ) {
		// is our page.
		if ( isset( $_GET['wp_otfd_file'] ) ) {
			$url_key = sanitize_text_field( wp_unslash( $_GET['wp_otfd_file'] ) );
		} else {
			$url_key = sanitize_text_field( wp_unslash( $_GET['file'] ) );
		}
		$link_controller = new LinkController();
		$download_data   = $link_controller->get_file_url_if_link_valid( $url_key );
		if ( isset( $download_data['link'] ) ) {
			if ( isset( $download_data['display_download_page'] ) && 1 == $download_data['display_download_page'] && ! isset( $_GET['fd'] ) ) {
				include OTFD_PLUGIN_PATH . '_views/fe-download.php';
				die;
			} elseif ( isset( $download_data['display_download_page'] ) && 2 == $download_data['display_download_page'] && ! isset( $_GET['fd'] ) ) {
				if ( isset( $download_data['download_page_wpotfd'] ) && 0 < $download_data['download_page_wpotfd'] && ! isset( $_GET['wp_otfd_page'] ) ) {
					wp_safe_redirect( add_query_arg( array( 'wp_otfd_file' => $_GET['wp_otfd_file'] ,'wp_otfd_page' => $download_data['download_page_wpotfd'] ), get_permalink( $download_data['download_page_wpotfd'] ) ) );
					exit;
				}
			} else {
				$link_controller->mark_file_url_as_downloaded( $url_key );
				$file_url  = $download_data['link'];
				$file_size = otfd_curl_get_file_size( $file_url );
				$file_name = $download_data['filename'];
				$file_name = str_replace( ' ', '%20', $file_name );

				$url_parts = pathinfo( $file_url );
				if ( ! isset( $file_name ) || empty( $file_name ) ) {
					$file_name = $url_parts['basename'];
				}
				$file_ext      = $url_parts['extension'];
				$ctype_default = 'application/octet-stream';
				$content_types = array(
					'exe'  => 'application/octet-stream',
					'zip'  => 'application/zip',
					'mp3'  => 'audio/mpeg',
					'mpg'  => 'video/mpeg',
					'avi'  => 'video/x-msvideo',
					'csv'  => 'text/csv',
					'doc'  => 'application/msword',
					'docx' => 'application/vnd.openxmlformats-officedocument.wordprocessingml.document',
					'epub' => 'epub',
					'gif'  => 'image/gif',
					'htm'  => 'text/html',
					'html' => 'text/html',
					'ico'  => 'image/x-icon',
					'ics'  => 'text/calendar',
					'jpeg' => 'image/jpeg',
					'jpg'  => 'image/jpeg',
					'png'  => 'image/png',
					'pdf'  => 'application/pdf',
					'json' => 'application/json',
					'ppt'  => 'application/vnd.ms-powerpoint',
					'pptx' => 'application/vnd.openxmlformats-officedocument.presentationml.presentation',
					'svg'  => 'image/svg+xml',
					'xls'  => 'application/vnd.ms-excel',
					'xlsx' => 'application/vnd.openxmlformats-officedocument.spreadsheetml.sheet',
					'xml'  => 'application/xml',
					'zip'  => 'application/zip',
				);
				if ( isset( $content_types[ $file_ext ] ) ) {
					$ctype = $content_types[ $file_ext ];
				} else {
					$ctype = $ctype_default;
				}

				header( 'Pragma: public' );
				header( 'Expires: -1' );
				header( 'Cache-Control: public, must-revalidate, post-check=0, pre-check=0' );
				header( 'Content-Type: ' . $ctype );
				header( 'Content-Disposition: attachment; filename="' . $file_name . '"' );
				if ( 0 < $file_size ) {
					header( 'Content-Length: ' . $file_size );
				}

				read_file_chunked( $file_url );
				die;
			}
		} else {
			if ( isset( $download_data['display_download_page'] ) && 1 == $download_data['display_download_page'] ) {
				include OTFD_PLUGIN_PATH . '_views/fe-downloaded.php';
				die;
			}
		}
	}
}

function read_file_chunked( $filename, $retbytes = true ) {
	$chunksize = 1 * ( 1024 * 1024 );
	$buffer    = '';
	$cnt       = 0;
	$handle    = fopen( str_replace( ' ', '%20', $filename ), 'rb' );
	// check if the file is valid.
	if ( false === $handle ) {
		return false;
	}
	while ( ! feof( $handle ) ) {
		$buffer = fread( $handle, $chunksize );
		echo $buffer;
		ob_flush();
		flush();
		if ( $retbytes ) {
			$cnt += strlen( $buffer );
		}
	}

	$status = fclose( $handle );
	if ( $retbytes && $status ) {
		return $cnt;
	}
	return $status;
}

add_action( 'init', 'check_if_wp_otfd_page' );
